
// JL: This code is based on the CodeProject article: 
// http://www.codeproject.com/KB/cpp/MemLeakDetect.aspx

#if !defined(MEMLEAKDETECT_H)
#define MEMLEAKDETECT_H
// This lot only works in _DEBUG builds, since the Windows APIs are debug-only.
#if defined(_DEBUG) && !defined(_WIN64)

#define _CRTDBG_MAP_ALLOC
#include <map>
#define _CRTBLD
#include <windows.h>

#include <..\crt\src\dbgint.h>
#include <dbghelp.h>
#include <crtdbg.h>

#include <boost/filesystem/path.hpp>

using namespace std;

// if you want to use the custom stackwalker otherwise
// comment this line out
#define MLD_CUSTOMSTACKWALK			1
//
#define MLD_MAX_NAME_LENGTH			256
#define MLD_MAX_TRACEINFO			256
#define MLD_TRACEINFO_EMPTY			_T("")
#define MLD_TRACEINFO_NOSYMBOL		_T("?(?)")

#ifdef  MLD_CUSTOMSTACKWALK
#define MLD_STACKWALKER				symStackTrace2
#else
#define MLD_STACKWALKER				symStackTrace
#endif

typedef DWORD ADDR;

class CMemLeakDetect
{
	public:

		typedef struct 	{
				ADDRESS				addrPC;
				ADDRESS				addrFrame;
			
		} STACKFRAMEENTRY;

		typedef struct { 
				void*				address;
				DWORD				size;
				TCHAR				fileName[MLD_MAX_NAME_LENGTH];
				DWORD				lineNumber;
				DWORD				occurance;
				STACKFRAMEENTRY		traceinfo[MLD_MAX_TRACEINFO];

		} AllocBlockInfo;

		typedef map<LPVOID, AllocBlockInfo>				KEYMAP;
		typedef map<LPVOID, AllocBlockInfo>::iterator	POSITION;
		typedef pair<LPVOID, AllocBlockInfo>			KEYVALUE;

		class CMapMem
		{
			public:

				KEYMAP			m_Map;
				POSITION		m_Pos;

				inline BOOL Lookup(LPVOID pAddr,  AllocBlockInfo& aInfo) { 

					m_Pos = m_Map.find(pAddr);
					//
					if (m_Pos == m_Map.end())
					{
						return FALSE;
					}
					//
					pAddr = m_Pos->first;
					aInfo = m_Pos->second;

					return TRUE;
				};

				inline POSITION end() { 

					return m_Map.end(); 
				};

				inline void RemoveKey(LPVOID pAddr) { 
					
					m_Map.erase(pAddr);
				};

				inline void RemoveAll() {
					m_Map.clear();
				};

				void SetAt(LPVOID pAddr, AllocBlockInfo& aInfo) {

					m_Map[pAddr] = aInfo;
				};

				inline POSITION GetStartPosition() { 
					POSITION pos = m_Map.begin(); 
					return pos;
				};

				inline void GetNextAssoc(POSITION& pos, LPVOID& rAddr, AllocBlockInfo& aInfo) {

					rAddr = pos->first;
					aInfo = pos->second;
					pos++;
				};

				void InitHashTable(int preAllocEntries, BOOL flag) 	{
					 preAllocEntries	= NULL;
					 flag				= NULL;
				};

		};

		CMemLeakDetect(const TCHAR * outputDir = NULL);
		~CMemLeakDetect();
		void Init();
		void End();
		void addMemoryTrace(void* addr,  DWORD asize,  TCHAR *fname, DWORD lnum);
		void redoMemoryTrace(void* addr,  void* oldaddr, DWORD asize,  TCHAR *fname, DWORD lnum);
		void removeMemoryTrace(void* addr, void* realdataptr);
		void cleanupMemoryTrace();
		void writeOutput(const TCHAR * szOutput);
		void dumpMemoryTrace();
		//

		CMapMem			 m_AllocatedMemoryList;
		DWORD memoccurance;
		bool  isLocked;
	//
	private:

		BOOL initSymInfo(TCHAR* lpUserPath);
		BOOL cleanupSymInfo();
		void symbolPaths( TCHAR* lpszSymbolPaths, UINT BufSizeTCHARs);
		void symStackTrace(STACKFRAMEENTRY* pStacktrace);
		void symStackTrace2(STACKFRAMEENTRY* pStacktrace);
		BOOL symFunctionInfoFromAddresses(ULONG fnAddress, ULONG stackAddress, TCHAR* lpszSymbol, UINT BufSizeTCHARs);
		BOOL symSourceInfoFromAddress(UINT address, TCHAR* lpszSourceInfo, UINT BufSizeTCHARs);
		BOOL symModuleNameFromAddress(UINT address, TCHAR* lpszModule, UINT BufSizeTCHARs);

		HANDLE				m_hProcess;
		PIMAGEHLP_SYMBOL	m_pSymbol;
		DWORD				m_dwsymBufSize;

	public:
		std::vector<std::basic_string<TCHAR> >	m_funcsToIgnore;
		boost::filesystem::path					m_outputFolder;
		

	private:
		std::string								m_outputPath;
		std::ofstream							m_output;
};

#else

// Dummy replacement for release and 64 bit builds.
class CMemLeakDetect
{
public:
	CMemLeakDetect(const TCHAR * = NULL)
	{
	}
};

#endif
#endif

// JL: Example instrumentation usage - in a loop that causes memory growth, 
// get stack traces of outstanding allocations after each iteration. Stack 
// traces get written to a sequence of text files.

//std::auto_ptr<CMemLeakDetect> leakDetectorPtr;
//while (true)
//{
//	leakDetectPtr.reset( new CMemLeakDetect() );
//
//	// Leaky code here
//	// ...
//	
//	// A new text file will be produced for each iteration.
//	leakDetectPtr.reset();
//}
